import 'package:flutter/material.dart';

import 'package:provider/provider.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

import '../../theme/senior_theme_data.dart';
import '../../repositories/theme_repository.dart';
import '../senior_elevated_element/senior_elevated_element_wiget.dart';
import '../senior_loading/senior_loading_widget.dart';

enum SeniorButtonSize { small, big }

enum SeniorButtonType { primary, secondary, ghost, danger }

class SeniorButton extends StatelessWidget {
  /// Create a button.
  ///
  /// The [label] and [onPressed] parameters are required.
  const SeniorButton({
    Key? key,
    this.busy = false,
    this.busyMessage,
    this.disabled = false,
    this.fullLength = false,
    required this.label,
    required this.onPressed,
    this.size = SeniorButtonSize.small,
    this.type = SeniorButtonType.primary,
  }) : super(key: key);

  /// Defines whether the button will be in a busy state.
  /// In this state, a circular progress indicator is displayed on the button.
  ///
  /// The default value is false.
  final bool busy;

  /// A message to be displayed when the button is in a busy state.
  /// It is displayed only when the button is in a busy state.
  final String? busyMessage;

  /// Defines if the button will be disabled.
  ///
  /// The default value is false.
  final bool disabled;

  /// Determines whether the button will take up all available side space.
  ///
  /// The default value is false.
  final bool fullLength;

  /// The text displayed inside the button.
  /// If a value is given for [busyMessage] and the button is in busy state, the busy message will be displayed.
  final String label;

  /// Function performed when the button is pressed.
  final Function() onPressed;

  /// The size of the button, which can be [SeniorButtonSize.small] and [SeniorButtonSize.big].
  ///
  /// The default value is [SeniorButtonSize.small].
  final SeniorButtonSize size;

  /// The button type according to the SDS definitions, it can be [SeniorButtonType.primary], [SeniorButtonType.secondary],
  /// [SeniorButtonType.ghost] and [SeniorButtonType.danger].
  ///
  /// The default value is [SeniorButtonType.primary].
  final SeniorButtonType type;

  /// Create an icon button.
  ///
  /// The [icon], [label] and [onPressed] parameters are required.
  factory SeniorButton.icon({
    Key? key,
    bool busy,
    bool disabled,
    bool fullLength,
    required IconData icon,
    required String label,
    required Function() onPressed,
    SeniorButtonSize size,
    SeniorButtonType type,
  }) = _SeniorButtonIcon;

  double _getButtonHeight() {
    switch (size) {
      case SeniorButtonSize.big:
        return 48.0;
      case SeniorButtonSize.small:
      default:
        return 40.0;
    }
  }

  Color _getFontColor(SeniorThemeData theme) {
    switch (type) {
      case SeniorButtonType.primary:
        return disabled
            ? theme.primaryButtonTheme?.style?.disabledFontColor ?? SeniorColors.primaryColor300
            : theme.primaryButtonTheme?.style?.fontColor ?? SeniorColors.pureWhite;
      case SeniorButtonType.secondary:
        return disabled
            ? theme.secondaryButtonTheme?.style?.disabledFontColor ?? SeniorColors.secondaryColor400
            : theme.secondaryButtonTheme?.style?.fontColor ?? SeniorColors.pureWhite;
      case SeniorButtonType.ghost:
        return disabled
            ? theme.ghostButtonTheme?.style?.disabledFontColor ?? SeniorColors.primaryColor100
            : theme.ghostButtonTheme?.style?.fontColor ?? SeniorColors.primaryColor;
      case SeniorButtonType.danger:
        return disabled
            ? theme.dangerButtonTheme?.style?.disabledFontColor ?? SeniorColors.pureWhite
            : theme.dangerButtonTheme?.style?.fontColor ?? SeniorColors.pureWhite;
    }
  }

  Color _getWaveColor(SeniorThemeData theme) {
    switch (type) {
      case SeniorButtonType.primary:
        return theme.primaryButtonTheme?.style?.waveColor ?? SeniorColors.primaryColor100;
      case SeniorButtonType.secondary:
        return theme.secondaryButtonTheme?.style?.waveColor ?? SeniorColors.secondaryColor100;
      case SeniorButtonType.ghost:
        return theme.ghostButtonTheme?.style?.waveColor ?? SeniorColors.neutralColor400;
      case SeniorButtonType.danger:
        return theme.dangerButtonTheme?.style?.waveColor ?? SeniorColors.manchesterColorRed100;
    }
  }

  TextStyle _getButtonTextStyle(SeniorThemeData theme) {
    return SeniorTypography.cta(
      color: _getFontColor(theme),
    ).copyWith(height: 0);
  }

  ButtonStyle _getButtonStyle(SeniorThemeData theme) {
    return ButtonStyle(
      overlayColor: MaterialStateProperty.all<Color>(_getWaveColor(theme)),
      backgroundColor: MaterialStateProperty.all<Color>(_getButtonColor(theme)),
      shape: MaterialStateProperty.all<OutlinedBorder>(
        RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(SeniorRadius.xbig),
        ),
      ),
      //A elevação é configurada pelo SeniorElevatedElement
      elevation: MaterialStateProperty.all<double>(0),
    );
  }

  Color _getButtonColor(SeniorThemeData theme) {
    switch (type) {
      case SeniorButtonType.primary:
        return disabled
            ? theme.primaryButtonTheme?.style?.disabledColor ?? SeniorColors.primaryColor100
            : theme.primaryButtonTheme?.style?.color ?? SeniorColors.primaryColor;
      case SeniorButtonType.secondary:
        return disabled
            ? theme.secondaryButtonTheme?.style?.disabledColor ?? SeniorColors.secondaryColor100
            : theme.secondaryButtonTheme?.style?.color ?? SeniorColors.secondaryColor;
      case SeniorButtonType.ghost:
        return disabled
            ? theme.ghostButtonTheme?.style?.disabledColor ?? Colors.transparent
            : theme.ghostButtonTheme?.style?.color ?? Colors.transparent;
      case SeniorButtonType.danger:
        return disabled
            ? theme.dangerButtonTheme?.style?.disabledColor ?? SeniorColors.manchesterColorRed300
            : theme.dangerButtonTheme?.style?.color ?? SeniorColors.manchesterColorRed;
    }
  }

  @override
  Widget build(BuildContext context) {
    final theme = Provider.of<ThemeRepository>(context).theme;

    return SizedBox(
      height: _getButtonHeight(),
      width: fullLength ? double.infinity : null,
      child: SeniorElevatedElement(
        child: ElevatedButton(
          key: key,
          child: busy
              ? Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    Padding(
                      padding: const EdgeInsets.only(
                        right: SeniorSpacing.small,
                      ),
                      child: SizedBox(
                        width: 20.0,
                        height: 20.0,
                        child: CircularProgressIndicator(
                          color: _getFontColor(theme),
                          strokeWidth: 3.0,
                        ),
                      ),
                    ),
                    Text(
                      busyMessage ?? label,
                      style: _getButtonTextStyle(theme),
                      overflow: TextOverflow.ellipsis,
                    ),
                  ],
                )
              : Text(
                  label,
                  style: _getButtonTextStyle(theme),
                  overflow: TextOverflow.ellipsis,
                ),
          onPressed: !disabled ? onPressed : null,
          style: _getButtonStyle(theme),
        ),
        borderRadius: SeniorRadius.xbig,
        elevation: disabled || type == SeniorButtonType.ghost ? SeniorElevations.dp0 : SeniorElevations.dp01,
      ),
    );
  }
}

class _SeniorButtonIcon extends SeniorButton {
  final IconData icon;

  _SeniorButtonIcon({
    Key? key,
    bool busy = false,
    bool disabled = false,
    bool fullLength = false,
    required this.icon,
    required String label,
    required Function() onPressed,
    SeniorButtonSize size = SeniorButtonSize.small,
    SeniorButtonType type = SeniorButtonType.primary,
  }) : super(
          key: key,
          busy: busy,
          disabled: disabled,
          fullLength: fullLength,
          label: label,
          onPressed: onPressed,
          size: size,
          type: type,
        );

  @override
  Widget build(BuildContext context) {
    final theme = Provider.of<ThemeRepository>(context).theme;
    return SizedBox(
      height: _getButtonHeight(),
      width: fullLength ? double.infinity : null,
      child: SeniorElevatedElement(
        child: ElevatedButton.icon(
          label: Text(
            label,
            style: _getButtonTextStyle(theme),
            overflow: TextOverflow.ellipsis,
          ),
          icon: busy
              ? const SeniorLoading(
                  size: SeniorIconSize.small,
                )
              : Icon(
                  icon,
                  size: SeniorIconSize.small,
                  color: _getFontColor(theme),
                ),
          onPressed: !disabled ? onPressed : null,
          style: _getButtonStyle(theme),
        ),
        borderRadius: SeniorRadius.xbig,
        elevation: disabled || type == SeniorButtonType.ghost ? SeniorElevations.dp0 : SeniorElevations.dp01,
      ),
    );
  }
}
