import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:provider/provider.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

import '../../../repositories/theme_repository.dart';
import '../senior_calendar_style.dart';
import 'senior_build_marked_day.dart';
import 'senior_default_marked_day.dart';
import 'senior_forms_enum.dart';

typedef void OnDayLongPressed(DateTime day);

class SeniorCalendarDay extends StatelessWidget {
  const SeniorCalendarDay({
    Key? key,
    this.disableDayPressed = false,
    this.disableNextDayMonthPressed = false,
    this.disablePrevDayMonthPressed = false,
    this.disableWeekendPressed = false,
    this.disableYesterdayPressed = false,
    required this.firstDayOfWeek,
    this.highlightToday = true,
    this.inactiveYesterdays = true,
    required this.index,
    required this.isNextMonthDay,
    required this.isPrevMonthDay,
    required this.isSelectable,
    required this.isSelectedDay,
    required this.isThisMonthDay,
    required this.isToday,
    required this.isWeekendDay,
    required this.isYesterday,
    required this.localeDate,
    required this.now,
    required this.onDayPressed,
    this.onDayLongPressed,
    this.style,
    this.buildMarkedDayWidget,
    this.showDefaultMarkedDay = false,
    this.showTriangleDefaultMarkedDay = false,
    this.showTriangleDownDefaultMarkedDay = false,
    this.showSquareDefaultMarkedDay = false,
    this.showCircleDefaultMarkedDay = false,
    this.DefaultMarkedseniorFormsEnum,
    this.isDayEventBuilderMarker = false,
    required this.colorCircleDefaultMarkedDay,
    required this.colorSquareDefaultMarkedDay,
    required this.colorTriangleDefaultMarkedDay,
    required this.colorTriangleDownDefaultMarkedDay,
  }) : super(key: key);

  ///This class is responsible for rendering calendar days.
  ///
  /// disables clicking on the day
  final bool disableDayPressed;

  /// disables clicking on the day of the next month
  final bool disableNextDayMonthPressed;

  /// disables the click on the day of the previous month
  final bool disablePrevDayMonthPressed;

  /// disables clicking on the weekend day
  final bool disableWeekendPressed;

  /// disables clicking on previous days
  final bool disableYesterdayPressed;

  /// first day of the week (0 = Sunday, 1 = Monday, etc) if 0, Sunday will be the first day of the week
  final int firstDayOfWeek;

  /// disable previous days
  final bool inactiveYesterdays;

  /// index of the day
  final int index;

  /// check if the day is the next month
  final bool isNextMonthDay;

  /// checks if the day is from the previous month
  final bool isPrevMonthDay;

  /// check if the day is selectable
  final bool isSelectable;

  /// checks if the day is selected
  final bool isSelectedDay;

  /// checks if the day is the current month
  final bool isThisMonthDay;

  /// checks if the day is today
  final bool isToday;

  /// checks if the day is the weekend
  final bool isWeekendDay;

  /// checks if the day is earlier
  final bool isYesterday;

  /// checks if the day and mark  highlights today
  final bool highlightToday;

  /// format the date
  final DateFormat localeDate;

  /// current date
  final DateTime now;

  /// function that is called when the day is clicked
  final OnDayLongPressed? onDayLongPressed;

  /// function that is called when the day is clicked for a long time
  final Function(DateTime)? onDayPressed;

  /// calendar style
  final SeniorCalendarStyle? style;

  /// widget that is rendered on the marked day
  final Widget? buildMarkedDayWidget;

  /// shows the default marked day, if standard marking is used
  final bool showDefaultMarkedDay;

  /// show triangle marking on default marked day
  final bool showTriangleDefaultMarkedDay;

  /// show triangle mark down on default marked day
  final bool showTriangleDownDefaultMarkedDay;

  /// show square marking on the default marked day
  final bool showSquareDefaultMarkedDay;

  /// show circle marking on default marked day
  final bool showCircleDefaultMarkedDay;

  /// list of enums that are used to mark days [SeniorFormsEnum.TRIANGLE,SeniorFormsEnum.CIRCLE,SeniorFormsEnum.SQUARE,SeniorFormsEnum.TRIANGLE_DOWN]
  final List<SeniorFormsEnum>? DefaultMarkedseniorFormsEnum;

  /// checks if the marked day is an event created by the user
  final bool isDayEventBuilderMarker;

  /// change color of the default marked day circle
  final Color? colorCircleDefaultMarkedDay;

  /// change color of the default marked day square
  final Color? colorSquareDefaultMarkedDay;

  /// change color of the default marked day triangle
  final Color? colorTriangleDefaultMarkedDay;

  /// change color of the triangle down of the default marked day
  final Color? colorTriangleDownDefaultMarkedDay;

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: const EdgeInsets.symmetric(vertical: SeniorSpacing.small / 2),
      child: Stack(
        children: [
          GestureDetector(
            onTap: () => _onDayPressed(
              picked: now,
              isWeekendDay: isWeekendDay,
              isYesterday: isYesterday,
              isPrevMonthDay: isPrevMonthDay,
              isNextMonthDay: isNextMonthDay,
            ),
            onLongPress: () => _onDayLongPressed(now),
            child: Align(
              alignment: Alignment.topCenter,
              child: Container(
                height: 36,
                width: 36,
                decoration: BoxDecoration(
                  color: _getBackgroundColor(context),
                  borderRadius: BorderRadius.circular(1000),
                ),
                child: Center(
                  child: Text(
                    '${now.day}',
                    semanticsLabel: now.day.toString(),
                    style: SeniorTypography.label(color: _getDayColor(context)),
                    maxLines: 1,
                  ),
                ),
              ),
            ),
          ),
          // ignore: prefer_const_constructors
          showDefaultMarkedDay
              ? SeniorDefaultMarkedDay(
                  showCircle: showCircleDefaultMarkedDay,
                  showSquare: showSquareDefaultMarkedDay,
                  showTriangle: showTriangleDefaultMarkedDay,
                  showTriangleDown: showTriangleDownDefaultMarkedDay,
                  colorSquareDefaultMarkedDay: colorSquareDefaultMarkedDay,
                  colorTriangleDefaultMarkedDay: colorTriangleDefaultMarkedDay,
                  colorTriangleDownDefaultMarkedDay: colorTriangleDownDefaultMarkedDay,
                  colorCircleDefaultMarkedDay: colorCircleDefaultMarkedDay,
                )
              : buildMarkedDayWidget != null
                  ? isDayEventBuilderMarker
                      ? SeniorBuildMarkedDay(buildMarkedDayWidget: buildMarkedDayWidget)
                      : Container()
                  : Container(),
        ],
      ),
    );
  }

  void _onDayPressed({
    required DateTime picked,
    required bool isWeekendDay,
    required bool isYesterday,
    required bool isPrevMonthDay,
    required bool isNextMonthDay,
  }) {
    if (disableDayPressed ||
        disableYesterdayPressed && isYesterday ||
        disableWeekendPressed && isWeekendDay ||
        disableNextDayMonthPressed && isNextMonthDay ||
        disablePrevDayMonthPressed && isPrevMonthDay) {
      return;
    }

    onDayPressed?.call(picked);
  }

  void _onDayLongPressed(DateTime picked) {
    onDayLongPressed?.call(picked);
  }

  Color _getDayColor(BuildContext context) {
    final theme = Provider.of<ThemeRepository>(context).theme;

    final Color todayColor =
        style?.todayColor ?? theme.calendarTheme?.style?.todayColor ?? SeniorColors.neutralColor800;

    final Color selectedDayColor =
        style?.selectedDayColor ?? theme.calendarTheme?.style?.selectedDayColor ?? SeniorColors.neutralColor800;

    final previousDaysMonthColor = style?.previousMonthDaysColor ??
        theme.calendarTheme?.style?.previousMonthDaysColor ??
        SeniorColors.neutralColor500;

    final nextDaysMonthColor =
        style?.nextMonthDaysColor ?? theme.calendarTheme?.style?.nextMonthDaysColor ?? SeniorColors.neutralColor500;

    final daysColor = style?.dayColor ?? theme.calendarTheme?.style?.dayColor ?? SeniorColors.neutralColor800;

    if (isSelectedDay) {
      return isToday ? todayColor : selectedDayColor;
    } else if (isToday) {
      return todayColor;
    } else if (isPrevMonthDay) {
      return previousDaysMonthColor;
    } else if (isNextMonthDay) {
      return nextDaysMonthColor;
    } else {
      return daysColor;
    }
  }

  Color _getBackgroundColor(BuildContext context) {
    final theme = Provider.of<ThemeRepository>(context).theme;

    final todayColor =
        style?.todayBackgroundColor ?? theme.calendarTheme?.style?.todayBackgroundColor ?? SeniorColors.neutralColor200;

    final previousMonthDayColor = style?.previousMonthDaysBackgroundColor ??
        theme.calendarTheme?.style?.previousMonthDaysBackgroundColor ??
        SeniorColors.neutralColor500;

    final nextMonthDayColor = style?.nextMonthDaysBackgroundColor ??
        theme.calendarTheme?.style?.nextMonthDaysBackgroundColor ??
        SeniorColors.neutralColor500;

    final selectedDayColor = style?.selectedDayBackgroundColor ??
        theme.calendarTheme?.style?.selectedDayBackgroundColor ??
        SeniorColors.primaryColor100;

    if (isSelectedDay) {
      return isToday && highlightToday ? todayColor : selectedDayColor;
    } else if (isToday && highlightToday) {
      return todayColor;
    } else if (isPrevMonthDay) {
      return previousMonthDayColor;
    } else if (isNextMonthDay) {
      return nextMonthDayColor;
    } else {
      return Colors.transparent;
    }
  }
}
