import 'package:flutter/material.dart';

import '../senior_badge/senior_badge_widget.dart';
import 'model/SeniorStatefulBadgeState.model.dart';

class SeniorStatefulBadge extends StatefulWidget {
  /// The background color of the badge.
  final Color backgroundColor;

  /// The background color displayed when the badge is disabled.
  ///
  /// If this parameter is not specified, the value of [backgroundColor] will
  /// be used.
  final Color? disabledBackgroundColor;

  /// The font color of the badge.
  final Color fontColor;

  /// The font color displayed when the badge is disabled.
  ///
  /// If this parameter is not specified, the value of [fontColor] will be used.
  final Color? disabledFontColor;

  /// The badge text.
  final String label;

  /// Whether the badge will be disabled or not.
  ///
  /// By default it is false, enabled.
  final bool disabled;

  /// The badge text when selected
  final String? selectedLabel;

  /// The background color of the badge when selected.
  final Color selectedBackgroundColor;

  /// The font color of the badge when selected.
  final Color selectedFontColor;

  /// The function callback when selected.
  final ValueSetter<SeniorStatefulBadgeCallbackModel> onSelect;

  /// Value sent to callback when selected.
  final String selectedValue;

  /// The shape of the badge.
  ///
  /// Possible values are [SeniorBadgeShape.chip] and [SeniorBadgeShape.pill].
  final SeniorBadgeShape shape;

  ///Create a Statefil Badge capable to return information on click
  ///
  ///
  ///The following parameters is required:
  /// - [backgroundColor]
  /// - [fontColor]
  /// - [label]
  /// - [shape]
  /// - [selectedBackgroundColor]
  /// - [selectedFontColor]
  /// - [onSelect]
  /// - [selectedValue]
  const SeniorStatefulBadge({
    super.key,
    required this.backgroundColor,
    this.disabledBackgroundColor,
    required this.fontColor,
    this.disabledFontColor,
    required this.label,
    this.disabled = false,
    required this.shape,
    this.selectedLabel,
    required this.selectedBackgroundColor,
    required this.selectedFontColor,
    required this.onSelect,
    required this.selectedValue,
  });

  ///Create a Statefil Badge with an icon capable to return information on click
  ///
  /// If [selectedIcon] was informed, it will be used when bedge select
  ///
  ///The following parameters is required:
  /// - [backgroundColor]
  /// - [fontColor]
  /// - [label]
  /// - [shape]
  /// - [selectedBackgroundColor]
  /// - [selectedFontColor]
  /// - [onSelect]
  /// - [selectedValue]
  /// - [icon]
  factory SeniorStatefulBadge.icon({
    required Color backgroundColor,
    required Color fontColor,
    required String label,
    required SeniorBadgeShape shape,
    required Color selectedBackgroundColor,
    required Color selectedFontColor,
    required ValueSetter onSelect,
    required String selectedValue,
    required IconData icon,
    IconData? selectedIcon,
  }) = _SeniorStatefulBadgeIcon;

  @override
  State<StatefulWidget> createState() => _SeniorStatefulBadgeState();
}

class _SeniorStatefulBadgeIcon extends SeniorStatefulBadge {
  final IconData icon;
  final IconData? selectedIcon;

  _SeniorStatefulBadgeIcon({
    required super.backgroundColor,
    required super.fontColor,
    required super.label,
    required super.shape,
    required super.selectedBackgroundColor,
    required super.selectedFontColor,
    required super.onSelect,
    required super.selectedValue,
    required this.icon,
    this.selectedIcon,
  }) : super();

  @override
  State<StatefulWidget> createState() => _SeniorStatefulBadgeIconState();
}

class _SeniorStatefulBadgeIconState extends State<_SeniorStatefulBadgeIcon> {
  late bool _selected;
  late Color _actualBackgroudColor;
  late Color _actualFontColor;
  late String _actualLabel;
  late IconData _actualIcon;

  @override
  void initState() {
    super.initState();
    _selected = false;
    _actualBackgroudColor = widget.backgroundColor;
    _actualFontColor = widget.fontColor;
    _actualLabel = widget.label;
    _actualIcon = widget.icon;
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () => setState(() {
        if (widget.disabled) {
          return;
        }
        _selected = !_selected;
        _actualBackgroudColor =
            _selected ? widget.selectedBackgroundColor : widget.backgroundColor;
        _actualFontColor =
            _selected ? widget.selectedFontColor : widget.fontColor;
        _actualLabel =
            _selected ? widget.selectedLabel ?? widget.label : widget.label;
        _actualIcon =
            _selected ? widget.selectedIcon ?? widget.icon : widget.icon;
        widget.onSelect(
            SeniorStatefulBadgeCallbackModel(widget.selectedValue, _selected));
      }),
      child: SeniorBadge.icon(
        disabled: widget.disabled,
        disabledBackgroundColor: widget.disabledBackgroundColor,
        disabledFontColor: widget.disabledFontColor,
        backgroundColor: _actualBackgroudColor,
        fontColor: _actualFontColor,
        label: _actualLabel,
        shape: widget.shape,
        icon: _actualIcon,
      ),
    );
  }
}

class _SeniorStatefulBadgeState extends State<SeniorStatefulBadge> {
  late bool _selected;
  late Color _actualBackgroudColor;
  late Color _actualFontColor;
  late String _actualLabel;

  @override
  void initState() {
    super.initState();
    _actualBackgroudColor = widget.backgroundColor;
    _actualFontColor = widget.fontColor;
    _actualLabel = widget.label;
    _selected = false;
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: () => setState(() {
        if (widget.disabled) {
          return;
        }
        _selected = !_selected;
        _actualBackgroudColor =
            _selected ? widget.selectedBackgroundColor : widget.backgroundColor;
        _actualFontColor =
            _selected ? widget.selectedFontColor : widget.fontColor;
        _actualLabel =
            _selected ? widget.selectedLabel ?? widget.label : widget.label;
        widget.onSelect(
            SeniorStatefulBadgeCallbackModel(widget.selectedValue, _selected));
      }),
      child: SeniorBadge(
        disabled: widget.disabled,
        disabledBackgroundColor: widget.disabledBackgroundColor,
        disabledFontColor: widget.disabledFontColor,
        backgroundColor: _actualBackgroudColor,
        fontColor: _actualFontColor,
        label: _actualLabel,
        shape: widget.shape,
      ),
    );
  }
}
